# Copyright (C) 2006 Tresys Technology, LLC
# License: refer to COPYING file for license information.
# Authors: Anand Patel <apatel@tresys.com>
#
# stripped down versin of audit2allow
#
import os
import output
import sys

import sepolgen.audit as audit
import sepolgen.policygen as policygen
import sepolgen.interfaces as interfaces
import sepolgen.objectmodel as objectmodel
import sepolgen.defaults as defaults

class AuditToPolicy:
		"""Main class that does parsing and outputing
		If error takes place than xml error code is written
		to stderr. Following is list of error codes and brief
		explanation.
		error code=1, something happened in overall program.
		error code=2, Message string was not received.
		error code=3, Interface info file was not received.
		error code=4, Interface info file can not be opened for read.
		"""
		VERSION = "%prog .1"

		def __init__(self):
			self.__options = None
			self.__parser = None
			self.__avs = None

		def __parse_options(self):
			from optparse import OptionParser

			parser = OptionParser(version=self.VERSION)
			parser.add_option("--interface-info", dest="interface_info", help="file name of interface information")
			parser.add_option("--slide-msg", dest="slide_msg", help="Audit message string")
			parser.add_option("--perm-map", dest="perm_map", help="file name of perm map")
			parser.add_option("--debug", dest="debug", action="store_true", default=False, help="leave generated modules for -M")

			options, args = parser.parse_args()

			if options.slide_msg is None:
				sys.stderr.write('<error code="2"/>')
				sys.exit(0)
			self.__options = options

		def __process_input(self):
			try:
				parser = audit.AuditParser()
			except:
				sys.exit(0)
				
			if self.__options.debug:
				sys.stderr.write('slide msg received\n')
			
			parser.parse_string(self.__options.slide_msg)

			self.__parser = parser

			self.__avs = self.__parser.to_access()



		# only called if the interface_info file was provided
		def __load_interface_info(self):
			# Load interface info file
			
			if self.__options.debug:
				sys.stderr.write("load_interface_info()\n")
			
			if os.path.exists(self.__options.interface_info):
				try:
					fd = open(self.__options.interface_info, 'r')
				except:
					sys.stderr.write('<error code="4"/>')
					sys.exit(0)
			else:
				sys.stderr.write('<error code="3"/>')
				sys.exit(0)
			

			if self.__options.debug:
				sys.stderr.write('generating interface info\n')
				
			ifs = interfaces.InterfaceSet()
			ifs.from_file(fd)
			fd.close()

			perm_maps = None

			# Also load perm maps
			if self.__options.perm_map:
				fn = self.__options.perm_map
			else:
				fn = defaults.perm_map()

			try:
				fd = open(fn)
				perm_maps = objectmodel.PermMappings()
				perm_maps.from_file(fd)
			except:
				return (ifs, perm_maps)

			return (ifs, perm_maps)

		def __output(self):
			g = policygen.PolicyGenerator()

			if self.__options.debug:
				sys.stderr.write('crated policygen\n')
				
			# Interface generation, if not provided skip it
			if self.__options.interface_info is not None:
				ifs, perm_maps = self.__load_interface_info()
				g.set_gen_refpol(ifs, perm_maps)

			# Generate the policy
			g.add_access(self.__avs)

			# Output
			writer = output.ModuleWriter()

			# stdout
			fd = sys.stdout
			xmldoc = writer.write(g.get_module())
			doc_root = xmldoc.documentElement
			nodelist = doc_root.childNodes
	
			for node in nodelist:
				sys.stdout.write(node.toxml())
				if self.__options.debug:
					sys.stderr.write(node.toxml())

		def main(self):
			try:
				self.__parse_options()
				self.__process_input()
				self.__output()
			except Exception, e:
				if self.__options.debug:
					sys.stderr.write(e);
				sys.exit(0)

if __name__ == "__main__":
	app = AuditToPolicy()
	app.main()
